/***************************************************************************
 *   Copyright (C) 2009 by Daniel Calviño Sánchez                          *
 *   kalvy@users.berlios.de                                                *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this library.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

/******************************************************************************
 *                                                                            *
 *                         DESCRIPTION OF THIS SCRIPT                         *
 *                                                                            *
 *****************************************************************************/

/*

DESCRIPTION
-----------
Replaces links to ogg videos with embedded players.

All the links with a child text node (that will likely contain the name of the
video file, altough not necessarily its full path) with class "videoLink" are
replaced.

The wrapper div uses the same id as the link, but its class is set to
"embeddedVideo".

The player used is wgOggPlayer, so the player script must be imported before
this script in the XHTML file.

The embedded player width, height and length parameter are got from the link id.
The alternative text for the still image is the same as the title of the link.
The video url is the video referenced by the link. However, the link url in
wgOggPlayer isn't set, so there is no "About file" option in the options box of
the player.

The link must have an id conforming to the following schema:
SomeIdentifier-WidthxHeight-LengthInSeconds

The identifier can be anything, and contain "-" or "x" characters. The width
and height must be specified in pixels, and the length in seconds.

Note that the width and length are those of the embedded player, not the video
itself. That is, the video can be 1024x768, but you may want to play it just in
480x360.


MOTIVATION
----------
The embedded video player used in Stephen Fry "Happy birthday to GNU" video
(http://www.gnu.org/fry/happy-birthday-to-gnu.html) has a little drawback: when
Javascript isn't enabled the video obviously does nothing. But it doesn't even
allow it to be downloaded.

In the GNU page there is a separated link to download the video, but I didn't
want to have the video and also an independent link, as the embedded player also
has its own download link (when Javascript is enabled).

So with this script, when Javascript isn't enabled a link to download the video
is shown, and when Javascript is enabled an embedded player is shown instead.


HOW DOES IT WORK?
-----------------
When embedVideos() is called, each link marked to be replaced (using the class
"videoLink") is removed and subtituted by a div that will contain the image and
button when the video is still, and the embedded player itself with the option
box when the video is running.

The div is filled with the still image and the button, and the ogg player script
changes between image and embedded player responding to user actions.

The values used in the embedded player and set in the image and button links are
got from the initial link to the video.


EMBEDDED VIDEO EXAMPLE
--------------------------------------
The XML below shows an example of a video link that can be replaced with an
embedded player.

<a href="files/ktutorial-l10n.ogg" title="Video of KTutorial localization"
   id="ktutorial-l10n.ogg-480x360-46">ktutorial-l10n.ogg</a>

The link would be replaced with:

<div id="ktutorial-l10n.ogg-480x360-46" class="embeddedVideo">

  <div>
    <a href="#dummyLink"
       onclick="wgOggPlayer.init(false,
                    {&quot;id&quot;: &quot;ktutorial-l10n.ogg-480x360-46&quot;,
                     &quot;videoUrl&quot;: &quot;files/ktutorial-l10n.ogg&quot;,
                     &quot;width&quot;: 480, &quot;height&quot;: 360,
                     &quot;length&quot;: 46,
                     &quot;isVideo&quot;: true});"
       title="Play ktutorial-l10n.ogg video">
        <img src="files/ktutorial-l10n.png" width="480" height="360"
             alt="Video of KTutorial localization"/>
    </a>
  </div>

  <div>
    <button onclick="wgOggPlayer.init(false,
                    {&quot;id&quot;: &quot;ktutorial-l10n.ogg-480x360-46&quot;,
                     &quot;videoUrl&quot;: &quot;files/ktutorial-l10n.ogg&quot;,
                     &quot;width&quot;: 480, &quot;height&quot;: 360,
                     &quot;length&quot;: 46,
                     &quot;isVideo&quot;: true});"
            style="width: 480px" title="Play ktutorial-l10n.ogg video">
      <img src="play.png" width="22" height="22"
           alt="Play ktutorial-l10n.ogg video"/>
    </button>
  </div>

</div>


TESTED WITH:
------------
-Mozilla Firefox 2.0.0.13 (should work with all the Gecko based browsers)
-Konqueror 3.5.6 (should work with all the KHTML based browsers)

*/


/******************************************************************************
 *                                                                            *
 *                          VIDEO EMBEDDER fUNCTIONS                          *
 *                                                                            *
 *****************************************************************************/

/**
 * The path to the "Play" icon to be used in the button.
 */
var playImageSrc = "images/play.png";

/**
 * Replaces all the links with class name videoLink with an embedded player for
 * the linked video.
 *
 * Only links that contain just a text as child node (which will be used in the
 * alternative text of the images and the button) are allowed.
 */
function embedVideos() {
  if (!scriptCanBeHandled()) {
    return;
  }

  var videosToEmbed = getElementsByClass(document.documentElement, "videoLink");

  var i;
  for (i=0; i<videosToEmbed.length; i++) {
    if (videosToEmbed[i].tagName.toLowerCase() == "a" &&
        videosToEmbed[i].childNodes.length == 1 &&
        videosToEmbed[i].firstChild.nodeType == Node.TEXT_NODE) {
      embedVideo(videosToEmbed[i]);
    }
  }
}

/**
 * Checks if the browser implementes all the needed ECMAScript bindings for the
 * script.
 * This script uses DOM Level 2 Core, DOM Level 2 HTML module and DOM Level 2
 * Events module.
 *
 * @return True if the browser conforms to the needed modules, false otherwise.
 */
function scriptCanBeHandled() {
  if (!(document.implementation.hasFeature("Core", "2.0") &&
        document.implementation.hasFeature("Events", "2.0") &&
        document.implementation.hasFeature("HTML", "2.0"))) {
    return false;
  }

  return true;
}

/**
 * Replaces the given link by an embedded ogg video player.
 *
 * @param linkElement The video link to be replaced.
 */
function embedVideo(linkElement) {
  var embeddedPlayerElement = createEmbeddedPlayer(linkElement);

  var parentElement = linkElement.parentNode;
  parentElement.replaceChild(embeddedPlayerElement, linkElement);
}

/**
 * Creates the wrapper div of the still image and the play button divs.
 * The wrapper div has the same id as the link to be replaced, and its class
 * name is set to "embeddedVideo".
 *
 * @param linkElement The video link to be replaced.
 * @return The wrapper div of the embedded player.
 */
function createEmbeddedPlayer(linkElement) {
  var mainDivElement = document.createElement("div");
  mainDivElement.id = linkElement.id;
  mainDivElement.className = "embeddedVideo";

  mainDivElement.appendChild(createStillImageDiv(linkElement));
  mainDivElement.appendChild(createButtonDiv(linkElement));

  return mainDivElement;
}

/**
 * Creates the div containing the still image of the video.
 * The still image is wrapped on a link that plays the video when clicked, just
 * like the play button.
 *
 * The image is got from the path to the video. The width and height are set to
 * those specified in the id of the link to be replaced.
 *
 * @param linkElement The video link to be replaced.
 * @return The div containing the still image.
 */
function createStillImageDiv(linkElement) {
  var divElement = document.createElement("div");

  var width = getWidthFromId(linkElement.id);
  var height = getHeightFromId(linkElement.id);

  var aElement = document.createElement("a");
  aElement.setAttribute("href", "#dummyLink");
  aElement.setAttribute("onclick", onClickValue(linkElement));
  aElement.setAttribute("title",
                        "Play " + linkElement.firstChild.data + " video");
  divElement.appendChild(aElement);

  var imgElement = document.createElement("img");
  var imageSrc = getImageSrcFromVideoSrc(linkElement.getAttribute("href"));
  imgElement.setAttribute("src", imageSrc);
  imgElement.setAttribute("width", width);
  imgElement.setAttribute("height", height);
  imgElement.setAttribute("alt", linkElement.getAttribute("title"));
  aElement.appendChild(imgElement);

  return divElement;
}

/**
 * Creates the div containing the button to play the video.
 * The button is set to a width equal to the width of the still image.
 *
 * The button contains also an image with a "Play" icon. The path to this icon
 * can be set in playImageSrc global variable.
 *
 * @param linkElement The video link to be replaced.
 * @return The div containing the play button.
 */
function createButtonDiv(linkElement) {
  var divElement = document.createElement("div");

  var width = getWidthFromId(linkElement.id);

  var buttonElement = document.createElement("button");
  buttonElement.setAttribute("onclick", onClickValue(linkElement));
  buttonElement.setAttribute("title",
                             "Play " + linkElement.firstChild.data + " video");
  buttonElement.setAttribute("style", "width: " + width + "px");
  divElement.appendChild(buttonElement);

  var imgElement = document.createElement("img");
  imgElement.setAttribute("src", playImageSrc);
  imgElement.setAttribute("width", "22");
  imgElement.setAttribute("height", "22");
  imgElement.setAttribute("alt",
                          "Play " + linkElement.firstChild.data + " video");
  buttonElement.appendChild(imgElement);

  return divElement;
}

/**
 * Returns the "onClick" attribute value to play a video.
 * The embedded video will have the same id as the replaced link element, and
 * the url of the video is the one pointed by the link. The width, height and
 * length are got from the id.
 *
 * @param linkElement The video link to be replaced.
 * @return The "onClick" attribute value to play a video.
 */
function onClickValue(linkElement) {
  var width = getWidthFromId(linkElement.id);
  var height = getHeightFromId(linkElement.id);
  var length = getLengthFromId(linkElement.id);

  var value = "wgOggPlayer.init(false, {";
  value += "\"id\": \"" + linkElement.id + "\",";
  value += "\"videoUrl\": \"" + linkElement.getAttribute("href") + "\",";
  value += "\"width\": " + width + ", \"height\": " + height + ","
  value += "\"length\": " + length + ",";
  value += "\"isVideo\": true});";
  return value;
}

/**
 * Gets the first number in the parameters encoded in the id.
 *
 * @param videoId The id of the video.
 * @return The width from the id.
 */
function getWidthFromId(videoId) {
  var parameters = videoId.substr(videoId.search("[0-9]+x[0-9]+-[0-9]+$"));

  return parameters.substring(0, parameters.indexOf("x"));
}

/**
 * Gets the second number in the parameters encoded in the id.
 *
 * @param videoId The id of the video.
 * @return The height from the id.
 */
function getHeightFromId(videoId) {
  var parameters = videoId.substr(videoId.search("[0-9]+x[0-9]+-[0-9]+$"));

  return parameters.substring(parameters.indexOf("x") + 1,
                              parameters.indexOf("-"));
}

/**
 * Gets the third number in the parameters encoded in the id.
 *
 * @param videoId The id of the video.
 * @return The length from the id.
 */
function getLengthFromId(videoId) {
  var parameters = videoId.substr(videoId.search("[0-9]+x[0-9]+-[0-9]+$"));

  return parameters.substr(parameters.indexOf("-") + 1);
}

/**
 * Returns the path to the still image of the video.
 * The path will be equal to the path of the video, but using a png image
 * instead of a video.
 *
 * @param videoSrc The path to the video.
 * @return The path to the still image of the video.
 */
function getImageSrcFromVideoSrc(videoSrc) {
  return videoSrc.substr(0, videoSrc.lastIndexOf('.')) + ".png";
}

/******************************************************************************
 *                                                                            *
 *                             GET ELEMENTS                                   *
 *                                                                            *
 *****************************************************************************/

/**
 * Returns all the child elements of "element" with the specified class name.
 * All the children are returned, not only direct children.
 *
 * @param element The element to get the elements from.
 * @param className The class name of the elements to get.
 * @return An array with all the elements of the specified class.
 */
function getElementsByClass(element, className) {
  var selectedElements = [];

  var allElements = [];
  getAllElements(element, allElements);

  var i;
  for (i=0; i<allElements.length; i++) {
    var elementClasses = allElements[i].className.split(" ");
    var added = false;
    var j;
    for (j=0; !added && j<elementClasses.length; j++) {
      if (elementClasses[j] == className) {
        selectedElements.push(allElements[i]);
        added = true;
      }
    }
  }

  return selectedElements;
}

/**
 * Appends all the child elements, and the element itself, of "element" in the
 * array "elementsList".
 * Children and the element itself are added only if they're true elements, not
 * if they're only nodes.
 *
 * All the children are returned, not only direct children.
 *
 * @param element The element to add its children and itself.
 * @param elementsList The list to append the elements to.
 */
function getAllElements(element, elementsList) {
  if (element.nodeType == Node.ELEMENT_NODE) {
    elementsList.push(element);
  }

  var childNodes = element.childNodes;
  var i;
  for (i=0; i<childNodes.length; i++) {
    getAllElements(childNodes[i], elementsList);
  }
}
